#include "main.h"

// Function called on module load
int initModuleEx(HWND hParent, HINSTANCE hInst, LPCSTR pszPath) {
	hwndParent = hParent;
	hInstance = hInst;

	WNDCLASS wc = { 0 };
	wc.lpfnWndProc = WndProc;
	wc.hInstance = hInst;
	wc.lpszClassName = szAppName;

	RegisterClass(&wc);

	hwndMain = CreateWindowEx(WS_EX_TOOLWINDOW, szAppName, NULL, WS_POPUP, 0, 0, 0, 0, NULL, NULL, hInst, NULL);
	SendMessage(hwndParent, LM_REGISTERMESSAGE, (WPARAM)hwndMain, (LPARAM)msgs);

	///
	// Add bang commands here
	///
	AddBangCommand("!Amp_Play", iTunes_Play_Bang);
	AddBangCommand("!Amp_Pause", iTunes_Pause_Bang);
	AddBangCommand("!Amp_Stop", iTunes_Stop_Bang);
	AddBangCommand("!Amp_Next", iTunes_Next_Bang);
	AddBangCommand("!Amp_Prev", iTunes_Prev_Bang);
	AddBangCommand("!Amp_VolumeUp", iTunes_VolumeUp_Bang);
	AddBangCommand("!Amp_VolumeDown", iTunes_VolumeDown_Bang);
	AddBangCommand("!Amp_Rew", iTunes_Rew_Bang);
	AddBangCommand("!Amp_Power", iTunes_Power_Bang);
	AddBangCommand("!Amp_PowerOn", iTunes_PowerOn_Bang);
	AddBangCommand("!Amp_PowerOff", iTunes_PowerOff_Bang);
	return 0;
}

// Function called on module unload
void quitModule(HINSTANCE hInst) {

	SendMessage(hwndParent, LM_UNREGISTERMESSAGE, (WPARAM)hwndMain, (LPARAM)msgs);

	DestroyWindow(hwndMain);
	UnregisterClass(szAppName, hInst);

	///
	// Remove bang commands here
	///
	RemoveBangCommand("!Amp_Play");
	RemoveBangCommand("!Amp_Pause");
	RemoveBangCommand("!Amp_Stop");
	RemoveBangCommand("!Amp_Next");
	RemoveBangCommand("!Amp_Prev");
	RemoveBangCommand("!Amp_VolumeUp");
	RemoveBangCommand("!Amp_VolumeDown");
	RemoveBangCommand("!Amp_Rew");
	RemoveBangCommand("!Amp_Power");
	RemoveBangCommand("!Amp_PowerOn");
	RemoveBangCommand("!Amp_PowerOff");
}

// Message processor
LRESULT CALLBACK WndProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam) {
	switch (msg) {
		case LM_GETREVID: {
			if (wParam == 0 || wParam == 1) {
				LPSTR buf = (LPSTR)lParam;
				strcpy(buf, szVersion);
				return strlen(buf);
			}
			return 0;
		}
		break;

		case LM_REFRESH: {
			// LiteStep has been refreshed.
		}
		break;

		case WM_TIMER: {
			switch (wParam) {
				case IDT_TIMER_PLAY: {
					KillTimer(hwndMain, IDT_TIMER_PLAY);
					iTunes_Play_Func();
				}
				break;
				case IDT_TIMER_PAUSE: {
					KillTimer(hwndMain, IDT_TIMER_PAUSE);
					iTunes_Pause_Func();
				}
				break;
				case IDT_TIMER_STOP: {
					KillTimer(hwndMain, IDT_TIMER_STOP);
					iTunes_Stop_Func();
				}
				break;
				case IDT_TIMER_NEXT: {
					KillTimer(hwndMain, IDT_TIMER_NEXT);
					iTunes_Next_Func();
				}
				break;
				case IDT_TIMER_PREV: {
					KillTimer(hwndMain, IDT_TIMER_PREV);
					iTunes_Prev_Func();
				}
				break;
				case IDT_TIMER_VOLUMEUP: {
					KillTimer(hwndMain, IDT_TIMER_VOLUMEUP);
					iTunes_VolumeUp_Func();
				}
				break;
				case IDT_TIMER_VOLUMEDOWN: {
					KillTimer(hwndMain, IDT_TIMER_VOLUMEDOWN);
					iTunes_VolumeDown_Func();
				}
				break;
				case IDT_TIMER_REWIND: {
					KillTimer(hwndMain, IDT_TIMER_REWIND);
					iTunes_Rew_Func();
				}
				break;
				case IDT_TIMER_POWERON: {
					KillTimer(hwndMain, IDT_TIMER_POWERON);
					iTunes_PowerOn_Func();
				}
				break;
				case IDT_TIMER_POWEROFF: {
					KillTimer(hwndMain, IDT_TIMER_POWEROFF);
					iTunes_PowerOff_Func();
				}
				break;
			}
		}
		break;
	}
	return DefWindowProc(hwnd, msg, wParam, lParam);
}

VOID CALLBACK MyTimerProc(HWND hwnd, UINT message, UINT idTimer, DWORD dwTime) {

}

void iTunes_Play_Bang(HWND hCaller, LPCSTR pszArgs) {
	SetTimer(hwndMain, IDT_TIMER_PLAY, timerWait, NULL);
}

void iTunes_Play_Func() {
	IiTunes* iITunes = NULL;
	HRESULT hRes = ::CoCreateInstance(CLSID_iTunesApp, NULL, CLSCTX_LOCAL_SERVER, IID_IiTunes, (PVOID *)&iITunes);
	if (hRes == S_OK) {
		iITunes->Play();
	}
	iITunes->Release();
}

void iTunes_Pause_Bang(HWND hCaller, LPCSTR pszArgs) {
	if (FindWindow("iTunes", "iTunes")) {
		SetTimer(hwndMain, IDT_TIMER_PAUSE, timerWait, NULL);
	}
}

void iTunes_Pause_Func() {
	IiTunes* iITunes = NULL;
	HRESULT hRes = ::CoCreateInstance(CLSID_iTunesApp, NULL, CLSCTX_LOCAL_SERVER, IID_IiTunes, (PVOID *)&iITunes);
	if (hRes == S_OK) {

		ITPlayerState playerState;
		iITunes->get_PlayerState(&playerState);
		if (playerState == ITPlayerStateStopped) {
			iITunes->Play();
		}
		else {
			iITunes->Pause();
		}
	}
	iITunes->Release();
}

void iTunes_Stop_Bang(HWND hCaller, LPCSTR pszArgs) {
	if (FindWindow("iTunes", "iTunes")) {
		SetTimer(hwndMain, IDT_TIMER_STOP, timerWait, NULL);
	}
}

void iTunes_Stop_Func() {
	IiTunes* iITunes = NULL;
	HRESULT hRes = ::CoCreateInstance(CLSID_iTunesApp, NULL, CLSCTX_LOCAL_SERVER, IID_IiTunes, (PVOID *)&iITunes);
	if (hRes == S_OK) {
		iITunes->Stop();
	}
	iITunes->Release();
}

void iTunes_Next_Bang(HWND hCaller, LPCSTR pszArgs) {
	if (FindWindow("iTunes", "iTunes")) {
		SetTimer(hwndMain, IDT_TIMER_NEXT, timerWait, NULL);
	}
}

void iTunes_Next_Func() {
	IiTunes* iITunes = NULL;
	HRESULT hRes = ::CoCreateInstance(CLSID_iTunesApp, NULL, CLSCTX_LOCAL_SERVER, IID_IiTunes, (PVOID *)&iITunes);
	if (hRes == S_OK) {
		iITunes->NextTrack();
	}
	iITunes->Release();
}

void iTunes_Prev_Bang(HWND hCaller, LPCSTR pszArgs) {
	if (FindWindow("iTunes", "iTunes")) {
		SetTimer(hwndMain, IDT_TIMER_PREV, timerWait, NULL);
	}
}

void iTunes_Prev_Func() {
	IiTunes* iITunes = NULL;
	HRESULT hRes = ::CoCreateInstance(CLSID_iTunesApp, NULL, CLSCTX_LOCAL_SERVER, IID_IiTunes, (PVOID *)&iITunes);
	if (hRes == S_OK) {
		iITunes->PreviousTrack();
	}
	iITunes->Release();
}

void iTunes_VolumeUp_Bang(HWND hCaller, LPCSTR pszArgs) {
	if (FindWindow("iTunes", "iTunes")) {
		SetTimer(hwndMain, IDT_TIMER_VOLUMEUP, timerWait, NULL);
	}
}

void iTunes_VolumeUp_Func() {
	IiTunes* iITunes = NULL;
	HRESULT hRes = ::CoCreateInstance(CLSID_iTunesApp, NULL, CLSCTX_LOCAL_SERVER, IID_IiTunes, (PVOID *)&iITunes);
	if (hRes == S_OK) {
		long vol = NULL;
		iITunes->get_SoundVolume(&vol);
		vol += 2;
		iITunes->put_SoundVolume(vol);
	}
	iITunes->Release();
}

void iTunes_VolumeDown_Bang(HWND hCaller, LPCSTR pszArgs) {
	if (FindWindow("iTunes", "iTunes")) {
		SetTimer(hwndMain, IDT_TIMER_VOLUMEDOWN, timerWait, NULL);
	}
}

void iTunes_VolumeDown_Func() {
	IiTunes* iITunes = NULL;
	HRESULT hRes = ::CoCreateInstance(CLSID_iTunesApp, NULL, CLSCTX_LOCAL_SERVER, IID_IiTunes, (PVOID *)&iITunes);
	if (hRes == S_OK) {
		long vol = NULL;
		iITunes->get_SoundVolume(&vol);
		vol -= 2;
		iITunes->put_SoundVolume(vol);
	}
	iITunes->Release();
}

void iTunes_Rew_Bang(HWND hCaller, LPCSTR pszArgs) {
	if (FindWindow("iTunes", "iTunes")) {
		SetTimer(hwndMain, IDT_TIMER_REWIND, timerWait, NULL);
	}
}

void iTunes_Rew_Func() {
	IiTunes* iITunes = NULL;
	HRESULT hRes = ::CoCreateInstance(CLSID_iTunesApp, NULL, CLSCTX_LOCAL_SERVER, IID_IiTunes, (PVOID *)&iITunes);
	if (hRes == S_OK) {
		iITunes->BackTrack();
	}
	iITunes->Release();
}

void iTunes_Power_Bang(HWND hCaller, LPCSTR pszArgs) {
	if (FindWindow("iTunes", "iTunes")) {
		SetTimer(hwndMain, IDT_TIMER_POWEROFF, timerWait, NULL);
	}
	else {
		SetTimer(hwndMain, IDT_TIMER_POWERON, timerWait, NULL);
	}
}

void iTunes_PowerOn_Bang(HWND hCaller, LPCSTR pszArgs) {
	if (!FindWindow("iTunes", "iTunes")) {
		SetTimer(hwndMain, IDT_TIMER_POWERON, timerWait, NULL);
	}
}

void iTunes_PowerOn_Func() {
	IiTunes* iITunes = NULL;
	HRESULT hRes = ::CoCreateInstance(CLSID_iTunesApp, NULL, CLSCTX_LOCAL_SERVER, IID_IiTunes, (PVOID *)&iITunes);
	if (hRes == S_OK) {
		// HACK: Have to call a function for iTunes to stay open - maybe use ShellExecute() to run?
		iITunes->Stop();
	}
	iITunes->Release();
}

void iTunes_PowerOff_Bang(HWND hCaller, LPCSTR pszArgs) {
	if (FindWindow("iTunes", "iTunes")) {
		SetTimer(hwndMain, IDT_TIMER_POWEROFF, timerWait, NULL);
	}
}

void iTunes_PowerOff_Func() {
	IiTunes* iITunes = NULL;
	HRESULT hRes = ::CoCreateInstance(CLSID_iTunesApp, NULL, CLSCTX_LOCAL_SERVER, IID_IiTunes, (PVOID *)&iITunes);
	if (hRes == S_OK) {
		iITunes->Quit();
	}
	iITunes->Release();
}
